#!/usr/bin/env perl
#
# File: pldif.pl
# Author: Dael Maselli - LNF - INFN
# Date: 07 Sep 2010
# Purpose: Parse and reformat ldif or ldapsearch output
#

use strict;
use warnings;

if ( !$ARGV[0] ) {
  print "usage: $0 { ldif-file | - } [ template-file | - ] [multivalue-separator]\n";
  exit 1;
}

( my $ldifPath  = $ARGV[0] ) =~ s/\n|\r//g;

my $templPath = '';
if ( defined $ARGV[1] ) {
  ( $templPath = $ARGV[1] ) =~ s/\n|\r//g;
}

if ( $ldifPath eq '-' && $templPath eq '-' ) {
  print "usage: $0 { ldif-file | - } [ template-file | - ]\n";
  print "error: only one STDIN!\n";
  exit 1;
}

my $jsep = ',';
if ( defined $ARGV[2] ) {
  ( $jsep = $ARGV[2] ) =~ s/\n|\r//g;
}

my $template = '';

if ( $templPath ) {
  open( TH, "<$templPath" );
  $template .= $_ while <TH>;
  close( TH );
}

my %entry;
my $attr;
my $val;
my $begin = 1;

open( LH, "<$ldifPath" );

while ( <LH> ) {

  s/\n|\r//g;
  next if /^#/;
  next if /^(version|search|result): /;

  if ( /^$/ ) {
    if ( $begin == 0 ) {
      $begin = 1;
      doStuff();
      %entry = ();
    }
    next;
  }

  if ( /^ (.+)$/ ) {
    $entry{$attr}[$#{$entry{$attr}}] .= $1;
  } else {
    ( $attr, $val ) = split /: /;
    push @{ $entry{$attr} }, $val;
  }

  $begin = 0;

}

close( LH );


sub doStuff {

  $entry{'dn'}[0]   =~ /^([^,]+),\s*(.+)$/;
  $entry{'_rdn'}[0] = $1;
  $entry{'_bdn'}[0] = $2;
  $entry{'_ldc'}[0] = uc $1 if $entry{'dn'}[0] =~ /^[^,]+,\s*ou=People,\s*dc=(\w+)/i;

  if ( defined $entry{'schacPersonalUniqueID'}[0] ) {
    ( $entry{'_cf'}[0] = $entry{'schacPersonalUniqueID'}[0] ) =~ s/urn:mace:terena.org:schac:personalUniqueID:it:CF://; ;
  }

  if ( defined $entry{'schacPersonalUniqueCode'}[0] ) {
    ( $entry{'_authKey'}[0] = $entry{'schacPersonalUniqueCode'}[0] ) =~ s/urn:mace:terena.org:schac:personalUniqueCode:it:infn.it:authKey://; ;
  }

  if ( $template ) {

    my $tmp = $template;
    $tmp  =~ s/%([a-zA-Z_-]+)%/join( $jsep, @{ $entry{$1} } ) if defined $entry{$1}/ge;
    print $tmp;

  } else {

    # Alcune informazioni - prendo solo il primo valore!
    print "#" x 80 ."\n";
    print "# RDN:     " . $entry{'_rdn'}[0] . "\n";
    print "# Base DN: " . $entry{'_bdn'}[0] . "\n";
    print "#" x 80 ."\n";

    # Stampa tutta la entry
    print "dn: ". $entry{'dn'}[0] ."\n" ;
    foreach my $k ( sort keys %entry ) {    
      next if ( $k eq 'dn' ) || $k =~ /^_/;
      for ( 0 .. $#{ $entry{$k} } ) {
    	print "$k: ". $entry{$k}[$_] ."\n" ;
      }
    }

    print "\n";

  }

}
